﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Assets.Scripts.Game
{
    public class Game : INotifyPropertyChanged
    {
        Player _player = new Player();
        public Player Player { get { return _player; } }

		int _layer = 0;
        public int Layer { get { return _layer; } set { _layer = value; } }

		List<float> _layerRadii;
		public float GetLayerRadius(int layer) {
			return _layerRadii[layer];
		}

		int _ladderlevel = -1;

		GameObject _playerObj;
		GameObject _playerObjPhys;

		public void Start() {
			_layerRadii = new List<float>();
			GameObject[] levels = GameObject.FindGameObjectsWithTag("Level");
			foreach (GameObject level in levels) {
				SpawnLevelPhysics(level);
			}
			_layerRadii.Sort();
			_playerObj = GameObject.FindWithTag("Player");
			SpawnPlayerPhysics(_playerObj);
		}

		void SpawnLevelPhysics(GameObject level) {
			CircleCollider2D csph = level.GetComponent<CircleCollider2D>();
			float radius = csph.radius;
			_layerRadii.Add(radius);
			var length = Mathf.PI * 2.0f * radius;
			var physobj = new GameObject(level.name + "_phys");
			var cbox = physobj.AddComponent("BoxCollider2D") as BoxCollider2D;
			cbox.center = new Vector2(0.0f, radius - 0.05f);
			cbox.size = new Vector2(3.0f * length, 0.1f);
			GameObject.Destroy(csph);

			var numChild = level.transform.childCount;
			for ( int i = 0 ; i < numChild ; ++i ) {
				Transform child = level.transform.GetChild(i);
				GameObject obj = child.gameObject;
				Collider2D coll = obj.GetComponent<Collider2D>();
				if ( coll != null ) {
					SpawnStaticPhysics(obj, radius);
				}
			}
		}

		void SpawnPlayerPhysics(GameObject player) {
			var r = player.transform.position.magnitude;
			var t = Mathf.Atan2(player.transform.position.y, player.transform.position.x) - Mathf.PI / 2.0f;
			//Debug.Log("Pos=" + transform.position + " r=" + r + " t=" + t);
			var position = new Vector3(t, r, 0.0f);
			var rotation = Quaternion.AngleAxis(Mathf.Rad2Deg * (t), Vector3.forward);
			_playerObjPhys = GameObject.Instantiate(player, position, rotation) as GameObject;
			_playerObjPhys.name = player.name + "_phys";
			GameObject.Destroy(player.GetComponent<Collider2D>());
			// how to remove rendering form phys obj ?
			PlayerCC cc = _playerObjPhys.AddComponent("PlayerCC") as PlayerCC;
			cc.target = player.transform;
			cc.radius = GetLayerRadius(0);
			cc.anim   = player.GetComponent<Animator>();
			cc.game   = this;
			Rigidbody2D body = _playerObjPhys.AddComponent("Rigidbody2D") as Rigidbody2D;
			body.fixedAngle = true;
			body.interpolation = RigidbodyInterpolation2D.Interpolate;
			GameObject.Destroy(_playerObjPhys.GetComponent<SpriteRenderer>());
		}
		
		void SpawnStaticPhysics(GameObject obj, float layerRadius) {
			var r = obj.transform.position.magnitude;
			var t = Mathf.Atan2(obj.transform.position.y, obj.transform.position.x) - Mathf.PI / 2.0f;
			if ( t < -Mathf.PI ) {
				t += Mathf.PI * 2.0f;
			}
			else if ( t > Mathf.PI ) {
				t -= Mathf.PI * 2.0f;
			}
			//Debug.Log("Pos=" + transform.position + " r=" + r + " t=" + t);
			var position = new Vector3(-t * layerRadius, r, 0.0f);
			var rotation = Quaternion.AngleAxis(Mathf.Rad2Deg * (t), Vector3.forward) * Quaternion.Inverse(obj.transform.rotation);
			var physobj = GameObject.Instantiate(obj, position, rotation) as GameObject;
			physobj.name = obj.name + "_phys";
			RemovePhysicsComponentsRecursive(obj);
			RemoveRenderComponentsRecursive(physobj);
		}

		void RemovePhysicsComponentsRecursive(GameObject obj) {
			foreach ( var c in obj.GetComponents<Collider2D>() )
				GameObject.Destroy(c);
			foreach ( var c in obj.GetComponents<PhysicsScriptBase>() )
				GameObject.Destroy(c);
			var numChild = obj.transform.childCount;
			for ( var i = 0 ; i < numChild ; ++ i ) {
				var child = obj.transform.GetChild(i).gameObject;
				RemovePhysicsComponentsRecursive(child);
			}
		}
		
		void RemoveRenderComponentsRecursive(GameObject obj) {
			foreach ( var c in obj.GetComponents<SpriteRenderer>() )
				GameObject.Destroy(c);
			foreach ( var c in obj.GetComponents<RenderScriptBase>() )
				GameObject.Destroy(c);
			var numChild = obj.transform.childCount;
			for ( var i = 0 ; i < numChild ; ++ i ) {
				var child = obj.transform.GetChild(i).gameObject;
				RemoveRenderComponentsRecursive(child);
			}
		}

		public void setAboveLadder(int ladderlevel) {
			_ladderlevel = ladderlevel;
		}
		
		public void unsetAboveLadder() {
			_ladderlevel = -1;
		}

		public void ChangeLayer(int delta, bool ladder) {

			if ( ladder ) {
				if ( _ladderlevel < 0 )
					return;
				if ( _ladderlevel == Layer && delta < 0 )
					return;
				if ( _ladderlevel + 1 == Layer && delta > 0 )
					return;
			}
			

			Debug.Log("Change layer: cur=" + Layer + " delta=" + delta);
			if ( Layer == 3 && delta > 0 ) { // THE END
				Debug.Log("ENDDDDD");
				GameObject camera = GameObject.Find("MainCamera"); //.GetComponent<Camera>();
				GameObject.Destroy(camera.GetComponent("CameraMotion"));
				GameObject.Destroy(camera.GetComponent("PostProcessing"));
				camera.transform.position = new Vector3(40.0f, 0.0f, 0.0f);
				camera.transform.rotation = Quaternion.AngleAxis(180.0f, Vector3.forward);
				return;
			}



			Debug.Log("lay=" + Layer + " de=" + delta);
			float oldradius = _layerRadii[Layer];
			Layer += delta;
			if ( Layer < 0 ) {
				Layer = 0;
			}
			else if ( Layer >= _layerRadii.Count ) {
				Layer = _layerRadii.Count - 1;
			}


			float newradius = _layerRadii[Layer];
			(_playerObjPhys.GetComponent("PlayerCC") as PlayerCC).wrapToLevel(oldradius, newradius);
            OnPropertyChanged("Layer");
			GameObject.Find("MainCamera").GetComponent<Camera>().orthographicSize = 1.5f + 0.1f * Layer;
		}

        public event PropertyChangedEventHandler PropertyChanged;
        void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
